﻿using System;
using System.Collections.Generic;
using System.Text;

namespace RowShamBow
{
    [Serializable]
    public struct Hand : IComparable
    {
        static Hand zero = new Hand();
        static Hand h剪刀 = new Hand(1);
        static Hand h石头 = new Hand(2);
        static Hand h布 = new Hand(3);

        private int t;
        private Hand(int t)
        {
            this.t = t;
        }
        /// <summary>
        /// 无效的出拳
        /// </summary>
        public static Hand Zero
        {
            get { return zero; }
        }
        public static Hand 剪刀
        {
            get { return h剪刀; }
        }

        public static Hand 石头
        {
            get { return h石头; }
        }

        public static Hand 布
        {
            get { return h布; }
        }
        /// <summary>
        /// 更强者
        /// </summary>
        public Hand Stronger
        {
            get
            {
                switch (t)
                {
                    case 1:     //剪刀
                        return h石头;
                    case 2:     //石头
                        return h布;
                    case 3:     //布
                        return h剪刀;
                    default:
                        throw new ApplicationException();
                }
            }
        }
        /// <summary>
        /// 更弱者
        /// </summary>
        public Hand Weaker
        {
            get
            {
                switch (t)
                {
                    case 1:     //剪刀
                        return h布;
                    case 2:     //石头
                        return h剪刀;
                    case 3:     //布
                        return h石头;
                    default:
                        throw new ApplicationException();
                }
            }
        }
        /// <summary>
        /// 两个中的强者
        /// </summary>
        public static Hand GetStronger(Hand a, Hand b)
        {
            int diff = (b.t - a.t + 3) % 3;
            if (diff == 2)
                return a;
            return b;
        }
        /// <summary>
        /// 两个中的弱者
        /// </summary>
        public static Hand GetWeaker(Hand a, Hand b)
        {
            int diff = (b.t - a.t + 3) % 3;
            if (diff == 2)
                return b;
            return a;
        }

        public int CompareTo(object obj)
        {
            if (obj is Hand)
                return ((this.t + 1) - ((Hand)obj).t + 3) % 3 - 1;
            throw new ApplicationException();
        }
        //判断是否相等
        public override bool Equals(object obj)
        {
            if (obj is Hand)
                return this.t == ((Hand)obj).t;
            return false;
        }

        // 重载操作符 == 
        public static bool operator ==(Hand a, Hand b)
        {
            return a.t == b.t;
        }

        // 重载操作符 != 
        public static bool operator !=(Hand a, Hand b)
        {
            return a.t != b.t;
        }

        // 重载操作符 >
        public static bool operator >(Hand a, Hand b)
        {
            return b.Equals(a.Weaker);
        }

        // 重载操作符 <
        public static bool operator <(Hand a, Hand b)
        {
            return b.Equals(a.Stronger);
        }

        // 重载操作符 >=
        public static bool operator >=(Hand a, Hand b)
        {
            return (a == b) || (a > b);
        }

        // 重载操作符 <=
        public static bool operator <=(Hand a, Hand b)
        {
            return (a == b) || (a < b);
        }

        public override string ToString()
        {
            switch (t)
            {
                case 1:
                    return "剪刀";
                case 2:
                    return "石头";
                case 3:
                    return " 布 ";
            }
            return "Zero";
        }

        public override int GetHashCode()
        {
            return t.GetHashCode();
        }
    }
}
