﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Security;
using System.Security.Permissions;
using System.Security.Policy;
using System.Text;
using System.Windows.Forms;
using RowShamBowUI;

namespace RowShamBow.UI
{
    public partial class Form1 : Form
    {
        #region 初始化
        public Form1()
        {
            InitializeComponent();
        }

        //选手的类型集合
        List<Type> playerTypes = new List<Type>();
        private void Form1_Load(object sender, EventArgs e)
        {
            lblResult.Text = string.Empty;
            LoadPlayers(playerTypes);
            ShowPlayers(playerTypes);
        }

        //装载选手
        private void LoadPlayers(List<Type> playerTypes)
        {
            //内置的选手
            playerTypes.Add(typeof(PlayerA));
            playerTypes.Add(typeof(PlayerB));
            playerTypes.Add(typeof(PlayerAKiller));
            playerTypes.Add(typeof(PlayerBKiller));

            DirectoryInfo di = new DirectoryInfo("players");
            if (!di.Exists)
                return;
            foreach (FileInfo fi in di.GetFiles("*.dll"))
            {
                Assembly asm = Assembly.LoadFile(fi.FullName);
                foreach (Type type in asm.GetTypes())
                {
                    if (!type.IsClass || type.IsAbstract || !type.IsPublic || !type.IsSubclassOf(typeof(Player)))
                        continue;
                    ConstructorInfo ci = type.GetConstructor(System.Type.EmptyTypes);
                    if (ci == null || !ci.IsPublic)
                        continue;
                    playerTypes.Add(type);
                }
            }
        }
        //显示选手
        private void ShowPlayers(List<Type> playerTypes)
        {
            foreach (Type playerType in playerTypes)
            {
                ComboBoxItem<Type> item = new ComboBoxItem<Type>(playerType.Name, playerType);
                cmbPlayer1.Items.Add(item);
                cmbPlayer2.Items.Add(item);
                checkedListBox1.Items.Add(item, true);
            }
            if (cmbPlayer1.Items.Count >= 1)
                cmbPlayer1.SelectedIndex = 0;
            if (cmbPlayer2.Items.Count == 1)
                cmbPlayer2.SelectedIndex = 0;
            else if (cmbPlayer2.Items.Count > 1)
                cmbPlayer2.SelectedIndex = 1;
        }

        #endregion 初始化

        #region 一对一比赛
        /// <summary>
        /// 一对一的比赛
        /// </summary>
        private void btnStart_Click(object sender, EventArgs e)
        {
            if (cmbPlayer1.SelectedIndex < 0)
                return;
            if (cmbPlayer2.SelectedIndex < 0)
                return;
            btnStart.Enabled = false;
            Type playerType1 = ((ComboBoxItem<Type>)cmbPlayer1.SelectedItem).Value;
            Type playerType2 = ((ComboBoxItem<Type>)cmbPlayer2.SelectedItem).Value;

            Player player1 = BuildPlayer(playerType1);
            Player player2 = BuildPlayer(playerType2);

            StringBuilder sb = new StringBuilder();
            int player1WinCount, player2WinCount, DeadHeatCount;
            Compete(player1, player2, sb, out player1WinCount, out player2WinCount, out DeadHeatCount);
            string player1Name = playerType1.Name == player1.ToString() ? playerType1.Name : playerType1.Name + "(" + player1.ToString() + ")";
            string player2Name = playerType2.Name == player2.ToString() ? playerType2.Name : playerType2.Name + "(" + player2.ToString() + ")";
            lblResult.Text = string.Format("{0,3} - {1}\r\n{2,3} - 平手\r\n{3,3} - {4}",
                 player1WinCount.ToString(), player1Name,
                 DeadHeatCount.ToString(),
                 player2WinCount.ToString(), player2Name);
            lblResult.Invalidate();
            txbRecord.Text = sb.ToString(0, sb.Length - 2);
            txbRecord.Invalidate();
            btnStart.Enabled = true;
        }
        /// <summary>
        /// 一局比赛(100次)
        /// </summary>
        /// <param name="player1"></param>
        /// <param name="player2"></param>
        /// <param name="sb">文字显示</param>
        /// <param name="player1WinCount">player1胜次数</param>
        /// <param name="player2WinCount">player2胜次数</param>
        /// <param name="DeadHeatCount">平手次数</param>
        /// <returns>比赛结果。小于0:player1胜，大于0:player2胜，等于0平</returns>
        private static int Compete(Player player1, Player player2, StringBuilder sb, out int player1WinCount, out int player2WinCount, out int DeadHeatCount)
        {
            Hand player1Last = Hand.Zero, player1Hand;
            Hand player2Last = Hand.Zero, player2Hand;
            player1WinCount = 0;
            player2WinCount = 0;
            DeadHeatCount = 0;

            for (int i = 0; i < 100; i++)
            {
                player1Hand = player1.GetNext(player2Last);
                player2Hand = player2.GetNext(player1Last);
                player1Last = player1Hand;
                player2Last = player2Hand;
                switch (player1Hand.CompareTo(player2Hand))
                {
                    case -1:
                        player2WinCount++;
                        if (sb != null)
                            sb.Append(string.Format("{0,2} {1} < {2}, {3,3}, {4,3}, {5,3}\r\n", i, player1Hand, player2Hand, player1WinCount, DeadHeatCount, player2WinCount));
                        break;
                    case 0:
                        DeadHeatCount++;
                        if (sb != null)
                            sb.Append(string.Format("{0,2} {1} = {2}, {3,3}, {4,3}, {5,3}\r\n", i, player1Hand, player2Hand, player1WinCount, DeadHeatCount, player2WinCount));
                        break;
                    case 1:
                        player1WinCount++;
                        if (sb != null)
                            sb.Append(string.Format("{0,2} {1} > {2}, {3,3}, {4,3}, {5,3}\r\n", i, player1Hand, player2Hand, player1WinCount, DeadHeatCount, player2WinCount));
                        break;
                    default:
                        throw new ApplicationException();
                }
            }
            return Math.Sign(player1WinCount - player2WinCount);
        }

        #endregion 一对一比赛

        #region 循环赛
        //循环赛
        private void btnRound_Click(object sender, EventArgs e)
        {
            btnRound.Enabled = false;
            this.Size = new Size(500, 550);
            txbRecord.Text = string.Empty;
            this.Invalidate();
            List<Type> types = new List<Type>();
            foreach (ComboBoxItem<Type> item in checkedListBox1.CheckedItems)
            {
                types.Add(item.Value);
            }
            RoundRobin<Type> rr = new RoundRobin<Type>(types);
            rr.Compete += new CompeteHandler<Type>(rr_Compete);
            StringBuilder sb = new StringBuilder();
            rr.LostMark = 0;
            rr.SameMark = 1;
            rr.WinMark = 3;
            rr.Start(sb);
            StringBuilder sbSumery = new StringBuilder();
            for (int i = 0; i < types.Count; i++)
            {
                sbSumery.Append(string.Format("{0,3} - {1}\r\n", rr.Marks[i], rr.Players[i].Name));
            }
            txbRecord.AppendText("得分：\r\n" + sbSumery.ToString());
            btnRound.Enabled = true;
        }
        /// <summary>
        /// 循环赛传出的一局比赛事件
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        int rr_Compete(object sender, CompeteEventArgs<Type> e)
        {
            //配好两位选手
            Player player1 = BuildPlayer(e.Player1);
            Player player2 = BuildPlayer(e.Player2);
            int player1WinCount, player2WinCount, DeadHeatCount;
            //调用比赛方法
            int ret = Compete(player1, player2, null, out player1WinCount, out player2WinCount, out DeadHeatCount);
            //显示本局比赛结果
            switch (ret)
            {
                case -1:
                    txbRecord.AppendText(string.Format("{0,15} < {1,-15}\r\n", e.Player1.Name, e.Player2.Name));
                    break;
                case 0:
                    txbRecord.AppendText(string.Format("{0,15} = {1,-15}\r\n", e.Player1.Name, e.Player2.Name));
                    break;
                case 1:
                    txbRecord.AppendText(string.Format("{0,15} > {1,-15}\r\n", e.Player1.Name, e.Player2.Name));
                    break;
            }
            return ret;
        }

        #endregion 循环赛

        #region 公共
        /// <summary>
        /// 创建选手
        /// </summary>
        private static Player BuildPlayer(Type playerType)
        {
            return Activator.CreateInstance(playerType) as Player;
        }
        #endregion 公共

    }

    class ComboBoxItem<T>
    {
        string text;
        T value;

        public T Value
        {
            get { return this.value; }
            set { this.value = value; }
        }

        public ComboBoxItem(string text, T value)
        {
            this.text = text;
            this.value = value;
        }

        public override string ToString()
        {
            return text;
        }
    }
}
