﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace RowShamBowUI
{
    public delegate int CompeteHandler<T>(object sender, CompeteEventArgs<T> e);
    /// <summary>
    /// 完整的循环赛机制
    /// </summary>
    /// <typeparam name="T"></typeparam>
    class RoundRobin<T>
    {
        public int WinMark { get; set; }    //胜一局得分
        public int LostMark { get; set; }   //负一局得分
        public int SameMark { get; set; }   //平一局得分
        public int[] Marks { get; private set; }    //每位选手的分数

        public T[] Players { get; private set; }    //选手集
        RoundPair roundPair;
        /// <summary>
        /// 一局比赛(100次)
        /// </summary>
        public event CompeteHandler<T> Compete;

        public RoundRobin(ICollection players)
        {
            this.Players = new T[players.Count];
            players.CopyTo(this.Players, 0);
            roundPair = new RoundPair(players.Count);
            Marks = new int[players.Count];
        }

        public void Start(StringBuilder sb)
        {
            int x, y, ret;
            while (roundPair.GetNext(out x, out y))
            {
                if (Compete != null)
                {
                    ret = Compete(this, new CompeteEventArgs<T>(Players[x], Players[y]));
                    switch (ret)
                    {
                        case -1:
                            Marks[y] += 3;
                            break;
                        case 0:
                            Marks[x]++;
                            Marks[y]++;
                            break;
                        case 1:
                            Marks[x] += 3;
                            break;
                    }
                }
            }
        }
    }

    public class CompeteEventArgs<T> : EventArgs
    {
        public T Player1 { get; private set; }
        public T Player2 { get; private set; }

        public CompeteEventArgs(T player1, T player2)
        {
            this.Player1 = player1;
            this.Player2 = player2;
        }
    }
}
